/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef _HEIGHT_FIELD_H
#define _HEIGHT_FIELD_H


#include "glib--glist.h"
#include "GlobalDefs.h"
#include "HeightFieldCore.h"
#include "tf_glistFlexArr.h"
#include <time.h>

#define PTYPE			float
#define TF_DEFAULT_FILENAME	"Untitled"

/*
 *  HeightField: a class to hold the basic info for 1 height field
 */ 
class HeightField : public HeightFieldCore
	{

	friend class HeightFieldDraw;
	friend class HeightFieldExport;
	friend class HeightFieldGenSSynth;
	friend class HeightFieldGenSubdiv;
	friend class HeightFieldModel;
	friend class HeightFieldOps;
	friend class HeightFieldReader;
	friend class HeightFieldSubset;
	friend class HeightFieldWhirlPinch;
	friend class HeightFieldWriter;

	public:
				HeightField(PTYPE *_hf_=NULL, int xsize=0, int ysize=0, char *name=NULL);
				~HeightField();

		// inline methods which return data 
		bool		isLoadedFromFile ()	{return d_loadedFromFile;}
		bool		isSaved ()		{return d_isSaved;}
		char		*getName ()		{return d_name;}
		Glib_GList	*getContourList ()	{return d_contourList;}
		time_t		getUpdateTime ()	{return d_updateTime;}
		time_t		getBackupTime ()	{return d_backupTime;}

		// other methods 
		static char	*getNewName ();
		PTYPE		*getCopyOfData ();
		void		backup ();
		void		restore ();
		void		destroyBackup ();
		void		setName (char *name);
		void		setSaved (bool isSaved);
		void		setLoadedFromFile (bool lff) {d_loadedFromFile=lff;}
		void		setContourList(TFGListFlexArr *l, time_t timestamp);

	protected:

		PTYPE		*d_hfBck,		// backup copy
				d_hfBckSealevel;	// waterlevel of backup
		char		*d_name;
		Glib_GList	*d_contourList;
		bool		d_loadedFromFile;
		bool		d_isSaved;
		time_t		d_updateTime,
				d_backupTime,
				d_clTimestamp;		// contour list 
		static int	s_HFcount;		// counter for versioning
	};


#endif // _HEIGHT_FIELD_H
