/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogTerrace.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "GuiDialogOAC.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogTerrace::TFDialogTerrace (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Terrace Dialog", 
				"Terrace: group elevation levels", 6, 4), 
		  d_vBoxList (TRUE, 5),
		  d_hBoxNLevels (TRUE, 5),
		  d_hBoxTightness (TRUE, 5),
		  d_hBoxAdjustSea (TRUE, 5),
		  d_adjNLevels (10, 1.0, 50.0, 0.01),
		  d_adjTightness (0.75, 0.0, 1.0, 0.01),
		  d_hsNLevels (d_adjNLevels),
		  d_hsTightness (d_adjTightness),
		  d_lblNLevels ("Number of Levels"),
		  d_lblTightness ("Tightness Factor"),
		  d_lblAdjustSea ("Adjust Waterlevel"),
		  d_optionsFrame ("Terrace Parameters")

{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogTerrace\n");
	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	sprintf (this->d_winTitle, "Terrace: %s", d_HF->getName());
	this->set_title (this->d_winTitle);
	this->iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_adjNLevels.value_changed, this, &TFDialogTerrace::updatePreviewCallback);
	connect_to_method (d_adjTightness.value_changed, this, &TFDialogTerrace::updatePreviewCallback);
	connect_to_method (d_btnAdjustSea.clicked, this, &TFDialogTerrace::updatePreviewCallback);
}


/*
 *  destructor: clean up 
 */
TFDialogTerrace::~TFDialogTerrace ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogTerrace\n");
}


/*
 *  insertOptions: insert the dialogs actual options 
 */
void TFDialogTerrace::insertOptions ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogTerrace called without Frame label",
		"TFDialogTerrace ::setHFobjs");

	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_hBoxNLevels.pack_start (d_lblNLevels, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblNLevels.show ();
	d_hsNLevels.set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsNLevels.set_digits (0);
	d_hsNLevels.set_draw_value (TRUE);
	d_hBoxNLevels.pack_end (d_hsNLevels, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsNLevels.show ();
	d_vBoxList.pack_start (d_hBoxNLevels, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxNLevels.show ();

	d_hBoxTightness.pack_start (d_lblTightness, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblTightness.show ();
	d_hsTightness.set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsTightness.set_digits (2);
	d_hsTightness.set_draw_value (TRUE);
	d_hBoxTightness.pack_end (d_hsTightness, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsTightness.show ();
	d_vBoxList.pack_start (d_hBoxTightness, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxTightness.show ();

	d_hBoxAdjustSea.pack_start (d_lblAdjustSea, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblAdjustSea.show ();
	d_btnAdjustSea.set_active (TRUE);
	d_hBoxAdjustSea.pack_end (d_btnAdjustSea, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_btnAdjustSea.show ();
	d_vBoxList.pack_start (d_hBoxAdjustSea, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxAdjustSea.show ();

	d_optionsFrame.add (d_vBoxList);
	this->d_table->attach (d_optionsFrame, 3, 6, 1, 3);
	d_vBoxList.show ();
	d_optionsFrame.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();

}



void TFDialogTerrace::buildDialogWindow ()
{
	this->set_usize (450, 225);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions ();
	this->fillActionArea ();
	
	this->show ();
}


/*
 *  buttonCallbackApply: transform the height field
 */
void TFDialogTerrace::buttonCallbackApply ()
{
	float		tightness=d_adjTightness.get_value (),
			value=d_adjNLevels.get_value ();
	bool		adjSealevel=d_btnAdjustSea.get_active();

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogTerrace::buttonCallbackApply");

	d_HFO->terrace (value, tightness, adjSealevel);
	d_HFD->draw ();

	// only do this if window stays open 
	if (d_applyHit)
		{
		this->setHFobjs (d_HF, d_HFD);
		updatePreviewCallback ();
		}

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a widget has been updated. 
 */
void TFDialogTerrace::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	float		tightness=d_adjTightness.get_value (),
			value=d_adjNLevels.get_value ();
	bool		adjSealevel=d_btnAdjustSea.get_active();

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogFill::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogFill::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFDPreview==NULL", "TFDialogFill::updatePreviewCallback");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	d_HFOPreview->terrace (value, tightness, adjSealevel);
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
}


